@@ -18,9 +18,40 @@ detibble<-function(data) {
 
 svydesign.default<-function(ids,probs=NULL,strata=NULL,variables=NULL, fpc=NULL,
                     data=NULL, nest=FALSE, check.strata=!nest,weights=NULL,pps=FALSE,
-                            variance=c("HT","YG"),...){
+                    calibrate.formula=NULL,variance=c("HT","YG"),
+                    na_weights=c("fail","warn","allow"), ...){
 
-  data<-detibble(data)
+    na.failsafe<-function(message="missing values in object"){
+      function(object,...){
+        if (NCOL(object)==0)
+          object
+        else {
+          ok <- complete.cases(object)
+          if (all(ok)) 
+            object
+          else stop(message)
+        }
+      }
+    }
+    
+    data<-detibble(data)
+
+    ## drop records with NA weights if that's what the user asks for
+    na_weights<-match.arg(na_weights)
+    if(inherits(weights,"formula")){
+        mf<-substitute(model.frame(weights, data=data, na.action=na.pass))
+        pre_weights<-eval.parent(mf)
+    } else if (!is.null(weights)) {
+        pre_weights<-data.frame(weights)
+    } else pre_weights<-rep(1,nrow(data))
+    if (any(is.na(pre_weights))){
+        if (na_weights=="fail"){
+            stop("missing values in weights and na_weights='fail'")
+        } else if(na_weights=="warn"){
+            warning("missing values in weights; records will be dropped")
+        }
+        data<-data[!is.na(pre_weights),]
+    }
     
   variance<-match.arg(variance)
   if(is.character(pps)){
@@ -54,18 +85,6 @@ svydesign.default<-function(ids,probs=NULL,strata=NULL,variables=NULL, fpc=NULL,
         
     }
 
-    na.failsafe<-function(message="missing values in object"){
-      function(object,...){
-        if (NCOL(object)==0)
-          object
-        else {
-          ok <- complete.cases(object)
-          if (all(ok)) 
-            object
-          else stop(message)
-        }
-      }
-    }
 
      na.id<-na.failsafe("missing values in `id'")
      if(inherits(ids,"formula")) {
@@ -92,6 +111,7 @@ svydesign.default<-function(ids,probs=NULL,strata=NULL,variables=NULL, fpc=NULL,
       probs<-eval.parent(mf)
     }
 
+
     na.weight<-na.failsafe("missing values in `weights'")
     if(inherits(weights,"formula")){
       mf<-substitute(model.frame(weights,data=data,na.action=na.weight))
@@ -104,7 +124,6 @@ svydesign.default<-function(ids,probs=NULL,strata=NULL,variables=NULL, fpc=NULL,
        else
          probs<-as.data.frame(1/as.matrix(weights))
      }
-
       
 
     na.strata<-na.failsafe("missing values in `strata'")
@@ -117,8 +136,11 @@ svydesign.default<-function(ids,probs=NULL,strata=NULL,variables=NULL, fpc=NULL,
         strata<-data.frame(strata=strata)
       has.strata<-TRUE
       for(i in 1:NCOL(strata)){ ##drop empty strata
-          if (is.factor(strata[[i]]))
+          if (is.factor(strata[[i]])) {
               strata[[i]]<-as.factor(as.character(strata[[i]]))
+          } else if (is.character(strata[[i]])){ ##coerce string to factor
+              strata[[i]]<-as.factor(strata[[i]])
+              }
           }
     } else {
       has.strata <-FALSE
@@ -204,13 +226,15 @@ svydesign.default<-function(ids,probs=NULL,strata=NULL,variables=NULL, fpc=NULL,
     }
 
   
-    if (is.numeric(probs) && length(probs)==1)
-      probs<-rep(probs, NROW(variables))
+    if (NROW(probs)==1 && NCOL(probs)==1 && is.numeric(probs[[1]]))
+      probs<-rep(probs[[1]], NROW(variables))
     
     if (length(probs)==0) probs<-rep(1,NROW(variables))
     
     if (NCOL(probs)==1) probs<-data.frame(probs)
 
+    if (NROW(probs)!=NROW(ids)) stop("lengths of prob/weights and ids don't match")
+    
     rval<-list(cluster=ids)
     rval$strata<-strata
     rval$has.strata<-has.strata
@@ -222,11 +246,18 @@ svydesign.default<-function(ids,probs=NULL,strata=NULL,variables=NULL, fpc=NULL,
     rval$call<-sys.call(-1)
     rval$pps<-pps
     class(rval)<-c("survey.design2","survey.design")
+    if (!is.null(calibrate.formula)){
+        rval<-recalibrate(rval, calibrate.formula)
+        rval$call<-sys.call(-1)
+    }
     rval
 }
 
 onestrat<-function(x,cluster,nPSU,fpc, lonely.psu,stratum=NULL,stage=1,cal=cal){
   
+  stratum_center <- attr(x, "recentering")
+  if (is.null(stratum_center)) stratum_center <- 0
+
   if (is.null(fpc))
       f<-rep(1,NROW(x))
   else{
@@ -250,16 +281,18 @@ onestrat<-function(x,cluster,nPSU,fpc, lonely.psu,stratum=NULL,stage=1,cal=cal){
     x<-rbind(x,matrix(0,ncol=ncol(x),nrow=nPSU-nrow(x)))
     scale<-rep(scale[1],NROW(x))
   }
+  
   if (lonely.psu!="adjust" || nsubset>1 ||
-      (nPSU>1 & !getOption("survey.adjust.domain.lonely")))
-      x<-sweep(x, 2, colMeans(x), "-")
+      (nPSU>1 & !getOption("survey.adjust.domain.lonely"))) {
+    stratum_center <- colMeans(x)
+  }
+  x<-sweep(x=x, MARGIN=2, STATS=stratum_center, FUN="-")
 
   if (nsubset==1 && nPSU>1 && getOption("survey.adjust.domain.lonely")){ 
       warning("Stratum (",stratum,") has only one PSU at stage ",stage)
       if (lonely.psu=="average" && getOption("survey.adjust.domain.lonely"))
           scale<-NA
     }
-  
   if (nPSU>1){
       return(crossprod(x*sqrt(scale)))
   } else {
@@ -278,16 +311,24 @@ onestrat<-function(x,cluster,nPSU,fpc, lonely.psu,stratum=NULL,stage=1,cal=cal){
 
 onestage<-function(x, strata, clusters, nPSU, fpc, lonely.psu=getOption("survey.lonely.psu"),stage=0, cal){
    if (NROW(x)==0)
-        return(matrix(0,NCOL(x),NCOL(x)))
-  stratvars<- tapply(1:NROW(x), list(factor(strata)), function(index){
-             onestrat(x[index,,drop=FALSE], clusters[index],
-             nPSU[index][1], fpc[index], ##changed from fpc[index][1], to allow pps(brewer)
-             lonely.psu=lonely.psu,stratum=strata[index][1], stage=stage,cal=cal)
-  })
-  p<-NCOL(x)
-  nstrat<-length(unique(strata))
-  nokstrat<-sum(sapply(stratvars,function(m) !any(is.na(m))))
-  apply(array(unlist(stratvars),c(p,p,length(stratvars))),1:2,sum,na.rm=TRUE)*nstrat/nokstrat
+       return(matrix(0,NCOL(x),NCOL(x)))
+   ## For the 'adjust' option for lonely PSUs,
+   ## recenter around mean from all PSUs in all strata
+   if (!is.null(lonely.psu) && lonely.psu == "adjust") {
+     n_PSUs_from_all_strata <- sum(tapply(X = nPSU, INDEX = as.numeric(strata), FUN = head, 1))
+     recentering <- colSums(x) / n_PSUs_from_all_strata
+   } else {
+     recentering <- 0
+   }
+   stratvars<- tapply(1:NROW(x), list(factor(strata)), function(index){
+       onestrat(x[index,,drop=FALSE] |> `attr<-`('recentering', recentering), clusters[index],
+                nPSU[index][1], fpc[index], ##changed from fpc[index][1], to allow pps(brewer)
+                lonely.psu=lonely.psu,stratum=strata[index][1], stage=stage,cal=cal)
+   })
+   p<-NCOL(x)
+   nstrat<-length(unique(strata))
+   nokstrat<-sum(sapply(stratvars,function(m) !any(is.na(m))))
+   apply(array(unlist(stratvars),c(p,p,length(stratvars))),1:2,sum,na.rm=TRUE)*nstrat/nokstrat
 }
 
 
@@ -296,7 +337,12 @@ svyrecvar<-function(x, clusters,  stratas, fpcs, postStrata=NULL,
                     one.stage=getOption("survey.ultimate.cluster")){
 
   x<-as.matrix(x)
-  cal<-NULL
+    cal<-NULL
+
+  if(isFALSE(fpcs$pps))
+      use_rcpp<-getOption("survey.use_rcpp")
+  else
+      use_rcpp<-FALSE  ## Brewer's method, doesn't work with the current rcpp version
   
   ## Remove post-stratum means, which may cut across clusters
   ## Also center the data using any "g-calibration" models
@@ -308,7 +354,8 @@ svyrecvar<-function(x, clusters,  stratas, fpcs, postStrata=NULL,
           x<-as.matrix(qr.resid(psvar$qr,x/psvar$w)*psvar$w)
         } else {
           ## G-calibration within clusters
-          cal<-c(cal, list(psvar))
+            cal<-c(cal, list(psvar))
+            use_rcpp<-FALSE
         }
       } else if (inherits(psvar, "raking")){
         ## raking by iterative proportional fitting
@@ -331,10 +378,15 @@ svyrecvar<-function(x, clusters,  stratas, fpcs, postStrata=NULL,
       }
     }
   }
-  
-  multistage(x, clusters,stratas,fpcs$sampsize, fpcs$popsize,
-             lonely.psu=getOption("survey.lonely.psu"),
-             one.stage=one.stage,stage=1,cal=cal)
+    if (use_rcpp) {
+        multistage_rcpp(x, clusters,stratas,fpcs$sampsize, fpcs$popsize,
+                        lonely.psu=getOption("survey.lonely.psu"),
+                        one.stage=one.stage,stage=1,cal=cal)
+    } else {
+        multistage(x, clusters,stratas,fpcs$sampsize, fpcs$popsize,
+                   lonely.psu=getOption("survey.lonely.psu"),
+                   one.stage=one.stage,stage=1,cal=cal)
+    }
 }
 
 multistage<-function(x, clusters,  stratas, nPSUs, fpcs,
@@ -376,10 +428,71 @@ multistage<-function(x, clusters,  stratas, nPSUs, fpcs,
 }
 
 
+## Ben Schneider's code to use C++. Should *agree* with multistage() except
+## for one known bug in lonely.psu
+##
+multistage_rcpp <- function(x, clusters,  stratas, nPSUs, fpcs,
+                            lonely.psu=getOption("survey.lonely.psu"),
+                            one.stage=FALSE, stage, cal){
+
+  lonely.psu <- switch(lonely.psu, 
+                       certainty = 'certainty',
+                       remove    = 'remove',
+                       adjust    = 'adjust',
+                       average   = 'average',
+                       fail      = 'fail',
+                       stop("Can't handle lonely.psu=",lonely.psu)
+  )
+  use_singleton_method_for_domains <- isTRUE(getOption("survey.adjust.domain.lonely"))
+
+  # Prepare the inputs to pass to Rcpp function
+  if (is.data.frame(clusters)) {
+    for (j in seq_len(ncol(clusters))) {
+      if (!is.numeric(clusters[[j]]))
+      clusters[[j]] <- as.numeric(as.factor(clusters[[j]]))
+    }
+  }
+  clusters <- as.matrix(clusters)
+
+  if (is.data.frame(stratas)) {
+    for (j in seq_len(ncol(stratas))) {
+      if (!is.numeric(stratas[[j]]))
+      stratas[[j]] <- as.numeric(as.factor(stratas[[j]]))
+    }
+  }
+  stratas <- as.matrix(stratas)
+
+  if (is.null(fpcs)) {
+    strata_pop_sizes <- matrix(Inf,
+                               nrow = nrow(nPSUs),
+                               ncol = ncol(nPSUs))
+  } else {
+    strata_pop_sizes <- as.matrix(fpcs)
+  }
+
+  strata_samp_sizes <- as.matrix(nPSUs)
+
+  # Call the Rcpp function
+  v <- arma_multistage(Y = as.matrix(x),
+                       samp_unit_ids = clusters,
+                       strata_ids = stratas,
+                       strata_samp_sizes = strata_samp_sizes,
+                       strata_pop_sizes = strata_pop_sizes,
+                       singleton_method = lonely.psu,
+                       use_singleton_method_for_domains = getOption("survey.adjust.domain.lonely"),
+                       use_only_first_stage = one.stage,
+                       stage = stage)
+
+  dimnames(v) <- list(colnames(x),colnames(x))
+  v
+}
+
+
+
 ## fpc not given are zero: full sampling.
 as.fpc<-function(df,strata,ids,pps=FALSE){
 
-  count<-function(x) length(unique(x))
+  count<-function(x) sum(!duplicated(x))
   
   sampsize<-matrix(ncol=ncol(ids),nrow=nrow(ids))
   for(i in 1:ncol(ids))
@@ -400,7 +513,18 @@ as.fpc<-function(df,strata,ids,pps=FALSE){
     cat("record",small[1,1]," stage ",small[1,2],": fpc=", fpc[small[1,,drop=FALSE]],"\n")      
     stop("Must have all fpc>=1 or all fpc<=1")
   }
-  
+
+  if (NCOL(sampsize)<NCOL(fpc)) stop("need at least as many id as fpc stages")
+    if (NCOL(sampsize)>NCOL(fpc)) {
+        tmp<-fpc   
+        if (ispopsize)
+            fpc<-matrix(Inf, nrow=NROW(sampsize), ncol=NCOL(sampsize))
+        else
+            fpc<-matrix(0, nrow=NROW(sampsize), ncol=NCOL(sampsize))
+        fpc[,1:NCOL(tmp)]<-tmp
+        warning("stages without fpc were treated as with replacement")
+    }
+    
   if (ispopsize){
     if(pps) stop("fpc must be specified as sampling fraction for PPS sampling")
     popsize<-fpc
@@ -436,7 +560,7 @@ as.fpc<-function(df,strata,ids,pps=FALSE){
    }
   
   
-  rval<-list(popsize=popsize, sampsize=sampsize)
+  rval<-list(popsize=popsize, sampsize=sampsize,pps=pps)
   class(rval)<-"survey_fpc"
   rval
 }
@@ -669,7 +793,14 @@ svytotal.survey.design2<-function(x,design, na.rm=FALSE, deff=FALSE,influence=FA
                                    postStrata=design$postStrata)
     attr(total,"statistic")<-"total"
     if (influence){
-        attr(total, "influence")<-x/design$prob
+         if (na.rm && (length(nas)>length(design$prob))) {
+             infl<-matrix(0,ncol=NCOL(x),nrow=length(nas))
+             infl[nas==0,]<-x/design$prob
+             attr(total, "influence")<-infl
+
+        }  else {
+            attr(total, "influence")<-x/design$prob
+            }
         }
 
     if (is.character(deff) || deff){
@@ -738,7 +869,13 @@ svymean.survey.design2<-function(x,design, na.rm=FALSE,deff=FALSE,influence=FALS
   attr(average,"var")<-v
     attr(average,"statistic")<-"mean"
     if (influence){
-        attr(average,"influence") <- x*pweights/psum
+        if (na.rm && (length(nas)>length(pweights))) {
+            infl<-matrix(0,ncol=NCOL(x),nrow=length(nas))
+            infl[nas==0,]<-x*pweights/psum
+            } else {
+                infl<-x*pweights/psum
+            }
+        attr(average,"influence") <- infl
     }
   class(average)<-"svystat"
   if (is.character(deff) || deff){
@@ -848,10 +985,19 @@ svyratio.survey.design2<-function(numerator=formula, denominator, design, separa
         attr(rval,"deff")<-deffs
     
     attr(rval,"call")<-sys.call()
-    if (influence)
-        attr(rval, "influence")<-r/design$prob
-    
+
+    if (influence){
+        if (na.rm && (length(nas)>length(design$prob))) {
+            infl<-matrix(0,ncol=NCOL(r),nrow=length(nas))
+            infl[nas==0,]<-r/design$prob
+        } else {
+                infl<-r/design$prob
+            }
+        attr(rval,"influence") <- infl
+    }
     class(rval)<-"svyratio"
     rval
     
   }
+
+
