@@ -3,7 +3,12 @@
 ## We use this form because it can be sparse and because it is easy to combine
 ## multistage and multiphase sampling.
 ##
+## For (stratified, cluster) simple random sampling,
+## Dcheck simplifies to (n/N-1)/(n-1) or -(1-p)/(n-1)
+##
 Dcheck_strat<-function(strata, prob){
+    if (is.character(strata))
+        strata<-as.factor(strata)
     strata<-as.numeric(strata) ## for ave()
     n<-length(strata)
     rval<-matrix(0, n,n)
@@ -29,8 +34,35 @@ Dcheck_multi<-function(id,strata,probs){
    rval
  }
 
-Dcheck_subset<-function(strata, subset, prob, withreplacement){
+
+## subsetting now happens in Dcheck_multi_subset
+## (no: need to know strata before subsetting to get sampsize.
+
+Dcheck_subset<-function(strata, prob,sampsize, withreplacement){
+    if (is.character(strata))
+        strata<-as.factor(strata)
     strata<-as.numeric(strata) ## for ave()
+    N<-length(strata)
+    n<-NROW(strata)
+    rval<-matrix(0, n,n)
+    #sampsize<-ave(strata,strata,FUN=length)
+    strats<-unique(strata)
+    if (!withreplacement){
+      for(strat in strats){
+        these <- strata == strat
+        ithese<-which(these)
+        rval[these,these]<- -(1-prob[ithese])/(sampsize[ithese]-1)
+      }
+    }
+    diag(rval)<-(1-prob)
+    rval
+}
+
+
+oldDcheck_subset<-function(strata, subset, prob, withreplacement){
+     if (is.character(strata))
+        strata<-as.factor(strata)
+   strata<-as.numeric(strata) ## for ave()
     N<-length(strata)
     n<-sum(subset)
     rval<-matrix(0, n,n)
@@ -47,7 +79,7 @@ Dcheck_subset<-function(strata, subset, prob, withreplacement){
     rval
   }
 
-Dcheck_multi_subset<-function(id,strata,subset,probs,withreplacement){
+oldDcheck_multi_subset<-function(id,strata,subset,probs,withreplacement){
    nstage<-NCOL(id)
    n<-sum(subset)
    rval<-matrix(0,n,n)
@@ -61,15 +93,47 @@ Dcheck_multi_subset<-function(id,strata,subset,probs,withreplacement){
        rval<- twophaseDcheck(rval, this_stage)
      }
    rval
+}
+
+
+Dcheck_multi_subset<-function(id,strata,subset,probs,withreplacement,phase1_is_iid){
+   nstage<-NCOL(id)
+   n<-sum(subset)
+   rval<-matrix(0,n,n)
+   if (phase1_is_iid)
+       return(as(diag(n),"sparseMatrix"))
+   sampsize<-NULL
+   for(stage in 1:nstage){
+       uid<-rep(FALSE,NROW(id))
+       uid[subset]<-!duplicated(id[subset,stage])
+       insubset<-rowsum(as.integer(subset),id[,stage],reorder=FALSE)>0
+       idx<-match(id[subset,stage],id[subset,stage][uid[subset]])
+       sampsize<-ave(as.numeric(id[,stage]),strata[,stage],FUN=function(i) length(unique(i)))
+       this_stage<-Dcheck_subset(strata[uid,stage],probs[uid,stage],sampsize, withreplacement)[idx,idx]
+       rval<- twophaseDcheck(rval, this_stage)
+     }
+   rval
  }
+
 twophaseDcheck<-function(Dcheck1,Dcheck2){
   as(-Dcheck1*Dcheck2+Dcheck1+Dcheck2,"sparseMatrix")
 }
 
 make_covmat<-function(design1,design2,subset){
-  withreplacement<-is.null(design1$fpc$popsize)
-  phase1<-Dcheck_multi_subset(design1$cluster, design1$strata, subset, design1$allprob, withreplacement)
-  phase2<-Dcheck_multi(design2$cluster, design2$strata, design2$allprob)
+    
+    if (!is.null(design1$dcheck))
+        phase1<-design1$dcheck[[1]]$dcheck
+    else{
+        withreplacement<-is.null(design1$fpc$popsize)
+        phase1<-Dcheck_multi_subset(design1$cluster, design1$strata, subset,
+                                    design1$allprob, withreplacement,
+                                    design1$phase1_is_iid)
+    }
+    if (!is.null(design2$dcheck))
+        phase2<-design2$dcheck[[1]]$dcheck
+    else
+        phase2<-Dcheck_multi(design2$cluster, design2$strata, design2$allprob)
+    
   dcheck<-twophaseDcheck(phase1,phase2)
   list(phase1=phase1,phase2=phase2,full=dcheck)
 }
@@ -78,12 +142,24 @@ make_covmat<-function(design1,design2,subset){
 ## Based on twophase(), so it computes some stuff that is no longer necessary.
 ## Will be pruned in the future.
 ##
-twophase2<-function(id,strata=NULL, probs=NULL, fpc=NULL,
-                   subset, data){
+twophase2<-function(id,strata=NULL, probs=NULL, fpc=NULL, pps=NULL,
+                    subset, data){
 
+    data<-detibble(data)
+
+    pps2<-NULL
+    if(!is.null(pps)){
+        if (!is.null(pps[[1]]))
+            stop("can't handle pps= at phase 1 yet")
+        pps2<-pps[[2]]
+    }
+
+    phase1_nulls<-is.null(probs[[1]]) && is.null(strata[[1]]) && is.null(fpc)[[1]] 
+    
   d1<-svydesign(ids=id[[1]],strata=strata[[1]],weights=NULL,
                 probs=probs[[1]],fpc=fpc[[1]],data=data)
 
+    d1$phase1_is_iid<-phase1_nulls && !any(duplicated(d1$cluster[,1]))
   if(inherits(subset,"formula"))
     subset<-eval.parent(model.frame(subset,data=data,na.action=na.pass))[[1]]
 
@@ -96,23 +172,30 @@ twophase2<-function(id,strata=NULL, probs=NULL, fpc=NULL,
                 probs=probs[[1]],fpc=fpc[[1]],data=data[subset,])
   d1s$prob<-d1$prob[subset]
   d1s$allprob<-d1$allprob[subset,,drop=FALSE]
-  
+
+
   ## work out phase-two fpc
-  if (is.null(fpc[[2]])){
-    complete.vars<-names(data)[apply(data, 2, function(v) all(!is.na(v)))]
-    if (all(c(all.vars(id[[2]]), all.vars(strata[[2]])) %in% complete.vars)){
-      dfpc<-svydesign(ids=id[[2]], strata=strata[[2]], data=data, probs=NULL)
-      popsize<-mapply(function(s,i) ave(!duplicated(i),s,FUN=sum), dfpc$strata, dfpc$cluster)
-      rm(dfpc)
+    if (is.null(fpc[[2]])){
+        
+        complete.vars<-names(data)[apply(data, 2, function(v) all(!is.na(v)))]
+        if (all(c(all.vars(id[[2]]), all.vars(strata[[2]])) %in% complete.vars)){
+            dfpc<-svydesign(ids=id[[2]], strata=strata[[2]], data=data, probs=NULL)
+            popsize<-mapply(function(s,i) ave(!duplicated(i),s,FUN=sum), dfpc$strata, dfpc$cluster)
+            rm(dfpc)
+        } else {
+            warning("Second-stage fpc not specified and not computable")
+            popsize<-NULL
+        }
+    } else popsize<-NULL
+
+    if (is.null(pps2)){
+        d2<-svydesign(ids=id[[2]], strata=strata[[2]], probs=probs[[2]],
+                      weights=NULL, fpc=fpc[[2]], data=data[subset,])
     } else {
-      warning("Second-stage fpc not specified and not computable")
-      popsize<-NULL
+        d2<-svydesign(ids=id[[2]], strata=strata[[2]],
+                      weights=NULL, fpc=probs[[2]], data=data[subset,],pps=pps2)
     }
-  } else popsize<-NULL
-
-  d2<-svydesign(ids=id[[2]], strata=strata[[2]], probs=probs[[2]],
-                weights=NULL, fpc=fpc[[2]], data=data[subset,])
-
+    
   ## ugly hack to get nicer labels
   if(!is.null(fpc[[2]])){
     d2call<-bquote(svydesign(ids=.(id[[2]]),strata=.(strata[[2]]), probs=.(probs[[2]]),
@@ -133,13 +216,14 @@ twophase2<-function(id,strata=NULL, probs=NULL, fpc=NULL,
 
   ## Add phase 2 fpc and probs if they were computed rather than specified.
   if (!is.null(popsize))
-    d2$fpc<-as.fpc(popsize[subset,,drop=FALSE],d2$strata,d2$cluster)
+      d2$fpc<-as.fpc(popsize[subset,,drop=FALSE],d2$strata,d2$cluster)
+    
   if(is.null(probs[[2]]) && !is.null(d2$fpc$popsize)){
     d2$allprob<-1/weights(d2$fpc,final=FALSE)
     d2$prob<-apply(as.data.frame(d2$allprob),1,prod)
   }
-  
   d2$variables<-NULL
+    
   deltacheck<-make_covmat(d1,d2, subset)
 
   rval<-list(phase1=list(full=d1,sample=d1s),
@@ -162,9 +246,9 @@ twophase2<-function(id,strata=NULL, probs=NULL, fpc=NULL,
       cm<-rval$phase1$full$cluster[,m]
       if (nunique(sa)!=nunique(sa[subset]))
           stop("Some phase-2 strata have zero sampling fraction")
-      rval$usu<-ave(cm[subset],sa[subset],FUN=nunique)/ave(cm,sa,FUN=nunique)[subset]
+      rval$usu<-ave(as.numeric(cm[subset]),sa[subset],FUN=nunique)/ave(as.numeric(cm),sa,FUN=nunique)[subset]
   } else {
-      rval$usu<-drop(with(rval$phase1$sample,ave(cluster[,m], strata[,m], FUN=nunique))/rval$phase1$full$fpc$sampsize[rval$subset])
+      rval$usu<-drop(with(rval$phase1$sample,ave(as.numeric(cluster[,m]), strata[,m], FUN=nunique))/rval$phase1$full$fpc$sampsize[rval$subset])
   }
 
   if (length(rval$phase1$sample$prob)==length(d2$prob))
@@ -173,7 +257,7 @@ twophase2<-function(id,strata=NULL, probs=NULL, fpc=NULL,
     rval$prob<-rep(Inf,length(rval$phase1$sample$prob))
     rval$prob[subset]<-rval$prob[subset]*d2$prob
   }
-  rval$call<-sys.call()
+  rval$call<-sys.call(-1)
   class(rval) <- c("twophase2","survey.design")
   rval
 }
@@ -207,6 +291,9 @@ print.summary.twophase2<-function(x,...,varnames=TRUE){
   invisible(x)
 }
 
+dimnames.twophase2<-function(x) dimnames(x$phase1$full$variables)
+
+
 twophase2var<-function(x,design){
   ## calibration is allowed at phase one or phase two,
   ## but not for clusters within a phase
@@ -588,6 +675,15 @@ calibrate.twophase2<-function(design, phase=2, formula, population,
             return(design)
         }
             
+        if(!inherits(formula,"formula")){
+            if (inherits(formula, "glm") || inherits(formula, "coxph") || inherits(formula, "lm")){
+                if (is.character(calfun))
+                    calfun<-switch(calfun,linear=cal.linear, raking=cal.raking, logit=cal.logit)
+                rval<-inf_calibrate(design, working_model=formula, add_mat=NULL,
+                                    calfun=calfun,...)
+                return(rval)
+            }
+        }
         if (missing(population) || is.null(population)){
             ## calibrate to phase 1 totals
             population<-colSums(model.matrix(formula,
@@ -640,7 +736,7 @@ estWeights.twophase2<-function(data, formula=NULL, working.model=NULL,...){
     model<-glm(ff, data=data$phase1$full$variables, family=binomial(),
                subset=!certainty, na.action=na.fail)
   } else {
-    xx<-estfun(working.model)
+    xx<-estfuns(working.model)
     model<-glm(data$subset~xx,family=binomial(), subset=!certainty, na.action=na.fail)
   }
   fitp<-as.numeric(certainty[data$subset])
